@@ -41,8 +41,8 @@ setGeneric("startIndex", function(x) standardGeneric("startIndex"))
 
 setGeneric("endIndex", function(x) standardGeneric("endIndex"))
 
-setMethod("elementLengths", "MIndex",
-    function(x) elementLengths(endIndex(x))
+setMethod("elementNROWS", "MIndex",
+    function(x) elementNROWS(endIndex(x))
 )
 
 setMethod("unlist", "MIndex",
@@ -71,14 +71,14 @@ setMethod("unlist", "MIndex",
     }
 )
 
-.fromMIndexToCompressedIRangesList <- function(from)
+.from_MIndex_to_CompressedIRangesList <- function(from)
 {
     ans <- relist(unlist(from, use.names=FALSE), from)
     names(ans) <- names(from)
     ans
 }
 
-setAs("MIndex", "CompressedIRangesList", .fromMIndexToCompressedIRangesList)
+setAs("MIndex", "CompressedIRangesList", .from_MIndex_to_CompressedIRangesList)
 
 extractAllMatches <- function(subject, mindex)
 {
@@ -143,7 +143,7 @@ setMethod("[[", "ByPos_MIndex",
 ###   > mindex[[6]] # Error in mindex[[6]] : subscript out of bounds
 ###   > startIndex(mindex)
 ###   > endIndex(mindex)
-###   > elementLengths(mindex)
+###   > elementNROWS(mindex)
 ###
 setMethod("startIndex", "ByPos_MIndex",
     function(x)
@@ -244,7 +244,7 @@ if (FALSE) {
   ###   > mindex[["aa"]] # Error in mindex[["aa"]] : pattern name ‘aa’ not found
   ###   > startIndex(mindex)
   ###   > endIndex(mindex)
-  ###   > elementLengths(mindex)
+  ###   > elementNROWS(mindex)
   ###
   setMethod("startIndex", "SparseMIndex",
     function(x)
@@ -267,10 +267,3 @@ if (FALSE) {
 
 }
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Old stuff (deprecated & defunct)
-###
-
-countIndex <- function(x) .Defunct("elementLengths")
-
