@@ -124,7 +124,9 @@ setReplaceMethod("subseq", "character",
     function(x, start=NA, end=NA, width=NA, value)
     {
         bands <- threebands(x, start=start, end=end, width=width)
-        paste(bands$left, value, bands$right, sep="")
+        ## Use x[] <- ... to preserve names and any other attribute.
+        x[] <- paste(bands$left, value, bands$right, sep="")
+        x
     }
 )
 
@@ -134,10 +136,14 @@ setReplaceMethod("subseq", "XStringSet",
     function(x, start=NA, end=NA, width=NA, value)
     {
         bands <- threebands(x, start=start, end=end, width=width)
-        if (is.null(value))
-            xscat(bands$left, bands$right)
-        else
-            xscat(bands$left, value, bands$right)
+        ## Use x[] <- ... to preserve class (endomorphism), names, metadata
+        ## columns, and any other attribute.
+        if (is.null(value)) {
+            x[] <- xscat(bands$left, bands$right)
+        } else {
+            x[] <- xscat(bands$left, value, bands$right)
+        }
+        x
     }
 )
 
@@ -340,7 +346,7 @@ setAs("ANY", "XStringSet",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "show" method.
+### The "show" and "showAsCell" methods.
 ###
 
 .namesW <- 20
@@ -422,38 +428,9 @@ setMethod("show", "XStringSet",
     }
 )
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Set Operations
-###
-
-.XStringSet.SetOperation <- function(x, y, FUN)
-{
-    x_seqtype <- seqtype(x)
-    if (seqtype(y) != x_seqtype)
-        stop("'x' and 'y' must be XStringSet objects containing ",
-             "sequences of the same type")
-    XStringSet(x_seqtype, FUN(as.character(unique(x)), as.character(unique(y))))
-}
-
-setMethod("union", c("XStringSet", "XStringSet"),
-    function(x, y, ...) .XStringSet.SetOperation(x, y, FUN = union)
-)
-setMethod("intersect", c("XStringSet", "XStringSet"),
-    function(x, y, ...) .XStringSet.SetOperation(x, y, FUN = intersect)
-)
-setMethod("setdiff", c("XStringSet", "XStringSet"),
-    function(x, y, ...) .XStringSet.SetOperation(x, y, FUN = setdiff)
-)
-setMethod("setequal", c("XStringSet", "XStringSet"),
-    function(x, y)
-    {
-        x_seqtype <- seqtype(x)
-        if (seqtype(y) != x_seqtype)
-            stop("'x' and 'y' must be XStringSet objects containing ",
-                 "sequences of the same type")
-        setequal(as.character(unique(x)), as.character(unique(y)))
-    }
+setMethod("showAsCell", "XStringSet",
+    function(object) 
+        vapply(object, toSeqSnippet, character(1), width=23L)
 )
 
 
