@@ -49,7 +49,10 @@ fasta.index <- function(filepath, nrec=-1L, skip=0L, seek.first.rec=FALSE,
     ans <- .Call2("fasta_index",
                   filexp_list, nrec, skip, seek.first.rec, lkup,
                   PACKAGE="Biostrings")
-    ans$filepath <- filepath[ans[ , "fileno"]]
+    ## 'expath' will usually be the same as 'filepath', except when 'filepath'
+    ## contains URLs which will be replaced by the path to the downloaded file.
+    expath <- vapply(filexp_list, attr, character(1), "expath", USE.NAMES=FALSE)
+    ans$filepath <- expath[ans[ , "fileno"]]
     ans
 }
 
@@ -119,7 +122,7 @@ fasta.seqlengths <- function(filepath, nrec=-1L, skip=0L, seek.first.rec=FALSE,
     offset <- ssorted_fai[ , "offset"]
     blockid <- recno - seq_along(recno)  # this block id is unique only within
                                          # a given file
-    is_first_in_block <- !S4Vectors:::duplicatedIntegerPairs(blockid, fileno)
+    is_first_in_block <- !duplicatedIntegerPairs(blockid, fileno)
     first_in_block_idx <- which(is_first_in_block)
     data.frame(fileno=fileno[first_in_block_idx],
                nrec=diff(c(first_in_block_idx, nrow(ssorted_fai) + 1L)),
@@ -409,7 +412,7 @@ saveXStringSet <- function(x, objname, dirpath=".",
 
 fasta.info <- function(...)
 {
-    .Deprecated("fasta.seqlengths")
+    .Defunct("fasta.seqlengths")
     fasta.seqlengths(...)
 }
 
