@@ -1,8 +1,6 @@
 #include "../inst/include/Biostrings_defines.h"
 #include <string.h>
 
-#define DEBUG_BIOSTRINGS 1
-
 #define INIT_STATIC_SYMBOL(NAME) \
 { \
 	if (NAME ## _symbol == NULL) \
@@ -12,8 +10,6 @@
 
 /* utils.c */
 
-SEXP debug_utils();
-
 void _init_ByteTrTable_with_lkup(
 	ByteTrTable *byte_tr_table,
 	SEXP lkup
@@ -61,15 +57,11 @@ int _get_twobit_signature_at(
 
 /* RoSeqs_utils.c */
 
-SEXP debug_RoSeqs_utils();
-
 RoSeqs _alloc_RoSeqs(int nelt);
 
 
 /* XString_class.c */
 
-SEXP debug_XString_class();
-
 const ByteTrTable *get_enc_byte2code(const char *classname);
 
 const ByteTrTable *get_dec_byte2code(const char *classname);
@@ -115,8 +107,6 @@ SEXP new_CHARACTER_from_XString(
 
 /* XStringSet_class.c */
 
-SEXP debug_XStringSet_class();
-
 int _get_XStringSet_length(SEXP x);
 
 SEXP _get_XStringSet_width(SEXP x);
@@ -188,8 +178,6 @@ SEXP XStringSet_xscat(SEXP args);
 
 /* XStringSet_io.c */
 
-SEXP debug_XStringSet_io();
-
 SEXP fasta_index(
 	SEXP filexp_list,
 	SEXP nrec,
@@ -366,6 +354,19 @@ SEXP DNAStringSet_translate(
 	SEXP if_ambig
 );
 
+/* replaceAt.c */
+
+SEXP XString_replaceAt(
+	SEXP x,
+	SEXP at,
+	SEXP value
+);
+
+SEXP XStringSet_replaceAt(
+	SEXP x,
+	SEXP at,
+	SEXP value
+);
 
 /* replace_letter_at.c */
 
@@ -407,8 +408,6 @@ SEXP XStringSetList_unstrsplit(
 
 /* SparseList_utils.c */
 
-SEXP debug_SparseList_utils();
-
 SEXP _SparseList_int2symb(int symb_as_int);
 
 int _SparseList_symb2int(SEXP symbol);
@@ -443,8 +442,6 @@ void _set_env_from_IntAEAE(
 
 /* match_reporting.c */
 
-SEXP debug_match_reporting();
-
 int _get_match_storing_code(const char *ms_mode);
 
 MatchBuf _new_MatchBuf(
@@ -501,8 +498,6 @@ MatchBuf *_get_internal_match_buf();
 
 /* MIndex_class.c */
 
-SEXP debug_MIndex_class();
-
 MIndex_holder _hold_MIndex(SEXP x);
 
 int _get_length_from_MIndex_holder(const MIndex_holder *x_holder);
@@ -535,8 +530,6 @@ SEXP ByPos_MIndex_combine(SEXP ends_listlist);
 
 /* lowlevel_matching.c */
 
-SEXP debug_lowlevel_matching();
-
 void _init_bytewise_match_tables();
 
 const BytewiseOpTable *_select_bytewise_match_table(int fixedP, int fixedS);
@@ -600,8 +593,6 @@ SEXP XStringSet_dist_hamming(SEXP x);
 
 /* match_pattern_boyermoore.c */
 
-SEXP debug_match_pattern_boyermoore();
-
 int _match_pattern_boyermoore(
 	const Chars_holder *P,
 	const Chars_holder *S,
@@ -612,8 +603,6 @@ int _match_pattern_boyermoore(
 
 /* match_pattern_shiftor.c */
 
-SEXP debug_match_pattern_shiftor();
-
 SEXP bits_per_long();
 
 void _match_pattern_shiftor(
@@ -627,8 +616,6 @@ void _match_pattern_shiftor(
 
 /* match_pattern_indels.c */
 
-SEXP debug_match_pattern_indels();
-
 void _match_pattern_indels(
 	const Chars_holder *P,
 	const Chars_holder *S,
@@ -640,8 +627,6 @@ void _match_pattern_indels(
 
 /* match_pattern.c */
 
-SEXP debug_match_pattern();
-
 void _match_pattern_XString(
 	const Chars_holder *P,
 	const Chars_holder *S,
@@ -702,8 +687,6 @@ SEXP XStringSet_vmatch_pattern(
 
 /* match_BOC.c */
 
-SEXP debug_match_BOC();
-
 SEXP match_BOC_preprocess(
 	SEXP s_xp,
 	SEXP s_offset,
@@ -741,8 +724,6 @@ SEXP match_BOC_exact(
 
 /* match_BOC2.c */
 
-SEXP debug_match_BOC2();
-
 SEXP match_BOC2_preprocess(
 	SEXP s_xp,
 	SEXP s_offset,
@@ -802,8 +783,6 @@ SEXP XStringViews_match_PWM(
 
 /* find_palindromes.c */
 
-SEXP debug_find_palindromes();
-
 SEXP find_palindromes(
 	SEXP x,
 	SEXP min_armlength,
@@ -889,13 +868,9 @@ void _BitMatrix_grow1rows(
 	const BitCol *bitcol
 );
 
-SEXP debug_BitMatrix();
-
 
 /* PreprocessedTB_class.c */
 
-SEXP debug_PreprocessedTB_class();
-
 SEXP _get_PreprocessedTB_tb(SEXP x);
 
 SEXP _get_PreprocessedTB_dups(SEXP x);
@@ -926,8 +901,6 @@ SEXP _get_ppdups_buf_asINTEGER();
 
 /* match_pdict_utils.c */
 
-SEXP debug_match_pdict_utils();
-
 TBMatchBuf _new_TBMatchBuf(
 	int tb_length,
 	int tb_width,
@@ -1001,8 +974,6 @@ void _match_pdict_all_flanks(
 
 /* match_pdict_Twobit.c */
 
-SEXP debug_match_pdict_Twobit();
-
 SEXP build_Twobit(
 	SEXP tb,
 	SEXP pp_exclude,
@@ -1019,8 +990,6 @@ void _match_Twobit(
 
 /* BAB_class.c */
 
-SEXP debug_BAB_class();
-
 SEXP IntegerBAB_new(SEXP max_nblock);
 
 int *_get_BAB_nblock_ptr(SEXP x);
@@ -1037,8 +1006,6 @@ SEXP _IntegerBAB_addblock(
 
 /* match_pdict_ACtree2.c */
 
-SEXP debug_match_pdict_ACtree2();
-
 SEXP ACtree2_nodebuf_max_nblock();
 
 SEXP ACtree2_nodeextbuf_max_nblock();
@@ -1082,8 +1049,6 @@ void _match_pdictACtree2(
 
 /* match_pdict.c */
 
-SEXP debug_match_pdict();
-
 SEXP match_PDict3Parts_XString(
 	SEXP pptb,
 	SEXP pdict_head,
