@@ -1,25 +1,11 @@
 /****************************************************************************
  *                   Basic manipulation of MIndex objects                   *
- *                           Author: Herve Pages                            *
+ *                            Author: H. Pag\`es                            *
  ****************************************************************************/
 #include "Biostrings.h"
 #include "IRanges_interface.h"
 #include "S4Vectors_interface.h"
 
-static int debug = 0;
-
-SEXP debug_MIndex_class()
-{
-#ifdef DEBUG_BIOSTRINGS
-	debug = !debug;
-	Rprintf("Debug mode turned %s in file %s\n",
-		debug ? "on" : "off", __FILE__);
-#else
-	Rprintf("Debug mode not available in file %s\n", __FILE__);
-#endif
-	return R_NilValue;
-}
-
 
 /****************************************************************************
  * C-level slot getters.
@@ -210,7 +196,8 @@ SEXP SparseMIndex_endIndex(SEXP x_ends_envir, SEXP x_width0, SEXP x_names, SEXP
 		UNPROTECT(1);
 	} else {
 		//poffsets_order = new_IntAE(nelt, 0, 0);
-		//get_order_of_int_array(poffsets->elts, nelt, 0, poffsets_order->elts, 0);
+		//get_order_of_int_array(poffsets->elts, nelt, 0, 0,
+		//		       poffsets_order->elts, 0);
 		//IntAE_set_nelt(poffsets_order) = nelt; /* = poffsets_order->_buflength */
 		PROTECT(ans = NEW_LIST(nelt));
 		PROTECT(ans_names = NEW_CHARACTER(nelt));
