@@ -1,5 +1,6 @@
 #include "Biostrings.h"
 #include "IRanges_interface.h"
+#include "S4Vectors_interface.h"
 #include <R_ext/Utils.h>        /* R_CheckUserInterrupt */
 
 #include <float.h>
@@ -8,6 +9,7 @@
 #define MAX(x, y) (x > y ? x : y)
 #define MIN(x, y) (x < y ? x : y)
 
+#define POSITIVE_INFINITY R_PosInf
 #define NEGATIVE_INFINITY R_NegInf
 #define MAX_BUF_SIZE      1048576
 
@@ -67,25 +69,25 @@ void print_AlignInfo(const struct AlignInfo *alignInfoPtr)
 	int string_len, i;
 	const char *string_seq, *c;
 
-	printf("- string: ");
+	Rprintf("- string: ");
 	string_len = alignInfoPtr->string.length;
-	string_seq = alignInfoPtr->string.seq;
+	string_seq = alignInfoPtr->string.ptr;
 	for (i = 0, c = string_seq; i < string_len; i++, c++)
-		printf("%c", *c);
-	printf("\n");
+		Rprintf("%c", *c);
+	Rprintf("\n");
 
-	printf("- quality: ");
+	Rprintf("- quality: ");
 	string_len = alignInfoPtr->quality.length;
-	string_seq = alignInfoPtr->quality.seq;
+	string_seq = alignInfoPtr->quality.ptr;
 	for (i = 0, c = string_seq; i < string_len; i++, c++)
-		printf("%c", *c);
-	printf("\n");
+		Rprintf("%c", *c);
+	Rprintf("\n");
 
-	printf("- endGap: %d\n", alignInfoPtr->endGap);
-	printf("- lengthMismatch: %d\n", alignInfoPtr->lengthMismatch);
-	printf("- lengthIndel: %d\n", alignInfoPtr->lengthIndel);
-	printf("- startRange: %d\n", alignInfoPtr->startRange);
-	printf("- widthRange: %d\n", alignInfoPtr->widthRange);
+	Rprintf("- endGap: %d\n", alignInfoPtr->endGap);
+	Rprintf("- lengthMismatch: %d\n", alignInfoPtr->lengthMismatch);
+	Rprintf("- lengthIndel: %d\n", alignInfoPtr->lengthIndel);
+	Rprintf("- startRange: %d\n", alignInfoPtr->startRange);
+	Rprintf("- widthRange: %d\n", alignInfoPtr->widthRange);
 	return;
 }
 
@@ -119,6 +121,100 @@ struct IndelBuffer {
 };
 void function4(struct IndelBuffer *);
 
+/* Traceback through the score matrices */
+static void traceback(const struct AlignBuffer *alignBufferPtr,
+		      char currTraceMatrix,
+		      struct AlignInfo *align1InfoPtr,
+		      struct AlignInfo *align2InfoPtr)
+{
+	int i, j;
+	char prevTraceMatrix = '?';
+	const char *sTraceMatrix = alignBufferPtr->sTraceMatrix;
+	const char *iTraceMatrix = alignBufferPtr->iTraceMatrix;
+	const char *dTraceMatrix = alignBufferPtr->dTraceMatrix;
+	const int nCharString1 = align1InfoPtr->string.length;
+	const int nCharString2 = align2InfoPtr->string.length;
+	const int nCharString1Minus1 = nCharString1 - 1;
+	const int nCharString2Minus1 = nCharString2 - 1;
+
+	//Rprintf("align1InfoPtr:\n");
+	//print_AlignInfo(align1InfoPtr);
+	//Rprintf("align2InfoPtr:\n");
+	//print_AlignInfo(align2InfoPtr);
+
+	i = nCharString1 - align1InfoPtr->startRange;
+	j = nCharString2 - align2InfoPtr->startRange;
+	while (currTraceMatrix != TERMINATION && i >= 0 && j >= 0) {
+		switch (currTraceMatrix) {
+		case INSERTION:
+			if (I_TRACE_MATRIX(i, j) != TERMINATION) {
+				if (j == nCharString2Minus1) {
+					align1InfoPtr->startRange++;
+				} else {
+					align1InfoPtr->widthRange++;
+					if (prevTraceMatrix != INSERTION) {
+						align2InfoPtr->startIndel[align2InfoPtr->lengthIndel] = nCharString2 - j;
+						align2InfoPtr->lengthIndel++;
+					}
+					align2InfoPtr->widthIndel[align2InfoPtr->lengthIndel - 1] += 1;
+				}
+			}
+			prevTraceMatrix = currTraceMatrix;
+			currTraceMatrix = I_TRACE_MATRIX(i, j);
+			i--;
+			break;
+		case DELETION:
+			if (D_TRACE_MATRIX(i, j) != TERMINATION) {
+				if (i == nCharString1Minus1) {
+					align2InfoPtr->startRange++;
+				} else {
+					align2InfoPtr->widthRange++;
+					if (prevTraceMatrix != DELETION) {
+						align1InfoPtr->startIndel[align1InfoPtr->lengthIndel] = nCharString1 - i;
+						align1InfoPtr->lengthIndel++;
+					}
+					align1InfoPtr->widthIndel[align1InfoPtr->lengthIndel - 1] += 1;
+				}
+			}
+			prevTraceMatrix = currTraceMatrix;
+			currTraceMatrix = D_TRACE_MATRIX(i, j);
+			j--;
+			break;
+	    	case SUBSTITUTION:
+			prevTraceMatrix = currTraceMatrix;
+			currTraceMatrix = S_TRACE_MATRIX(i, j);
+			if (currTraceMatrix != TERMINATION) {
+				align1InfoPtr->widthRange++;
+				align2InfoPtr->widthRange++;
+			}
+			if (currTraceMatrix != TERMINATION
+			 && align1InfoPtr->string.ptr[nCharString1Minus1 - i] != align2InfoPtr->string.ptr[nCharString2Minus1 - j]) {
+				align1InfoPtr->mismatch[align1InfoPtr->lengthMismatch] = nCharString1 - i;
+				align2InfoPtr->mismatch[align2InfoPtr->lengthMismatch] = nCharString2 - j;
+				align1InfoPtr->lengthMismatch++;
+				align2InfoPtr->lengthMismatch++;
+			}
+			i--;
+			j--;
+			break;
+		default:
+			error("unknown traceback code %d", currTraceMatrix);
+			break;
+		}
+	}
+
+	const int offset1 = align1InfoPtr->startRange - 1;
+	if (offset1 > 0 && align1InfoPtr->lengthIndel > 0) {
+		for (i = 0; i < align1InfoPtr->lengthIndel; i++)
+			align1InfoPtr->startIndel[i] -= offset1;
+	}
+	const int offset2 = align2InfoPtr->startRange - 1;
+	if (offset2 > 0 && align2InfoPtr->lengthIndel > 0) {
+		for (j = 0; j < align2InfoPtr->lengthIndel; j++)
+			align2InfoPtr->startIndel[j] -= offset2;
+	}
+	return;
+}
 
 /* Returns the score of the optimal pairwise alignment */
 static double pairwiseAlignment(
@@ -141,10 +237,6 @@ static double pairwiseAlignment(
 {
 	int i, j, iMinus1, jMinus1;
 
-	//printf("align1InfoPtr:\n");
-	//print_AlignInfo(align1InfoPtr);
-	//printf("align2InfoPtr:\n");
-	//print_AlignInfo(align2InfoPtr);
 	/* Step 1:  Get information on input XString objects */
 	const int nCharString1 = align1InfoPtr->string.length;
 	const int nCharString2 = align2InfoPtr->string.length;
@@ -155,9 +247,9 @@ static double pairwiseAlignment(
 	if (nCharString1 < 1 || nCharString2 < 1) {
 		double zeroCharScore;
 		if (nCharString1 >= 1 && align1InfoPtr->endGap)
-			zeroCharScore = gapOpening + nCharString1 * gapExtension;
+			zeroCharScore = - gapOpening - nCharString1 * gapExtension;
 		else if (nCharString2 >= 1 && align2InfoPtr->endGap)
-			zeroCharScore = gapOpening + nCharString2 * gapExtension;
+			zeroCharScore = - gapOpening - nCharString2 * gapExtension;
 		else
 			zeroCharScore = 0.0;
 		align1InfoPtr->lengthMismatch = 0;
@@ -172,14 +264,14 @@ static double pairwiseAlignment(
 	float *currMatrix = alignBufferPtr->currMatrix;
 	float *prevMatrix = alignBufferPtr->prevMatrix;
 	CURR_MATRIX(0, 0) = 0.0;
-	CURR_MATRIX(0, 1) = (align2InfoPtr->endGap ? gapOpening : 0.0);
+	CURR_MATRIX(0, 1) = (align2InfoPtr->endGap ? - gapOpening : 0.0);
 	for (i = 1, iMinus1 = 0; i <= nCharString1; i++, iMinus1++) {
 		CURR_MATRIX(i, 0) = NEGATIVE_INFINITY;
 		CURR_MATRIX(i, 1) = NEGATIVE_INFINITY;
 	}
 	if (align1InfoPtr->endGap) {
 		for (i = 0; i <= nCharString1; i++)
-			CURR_MATRIX(i, 2) = gapOpening + i * gapExtension;
+			CURR_MATRIX(i, 2) = - gapOpening - i * gapExtension;
 	} else {
 		for (i = 0; i <= nCharString1; i++)
 			CURR_MATRIX(i, 2) = 0.0;
@@ -203,7 +295,7 @@ static double pairwiseAlignment(
 	const int noEndGap1 = !align1InfoPtr->endGap;
 	const int noEndGap2 = !align2InfoPtr->endGap;
 	const float gapOpeningPlusExtension = gapOpening + gapExtension;
-	const float endGapAddend = (align2InfoPtr->endGap ? gapExtension : 0.0);
+	const float endGapAddend = (align2InfoPtr->endGap ? - gapExtension : 0.0);
 	float *tempMatrix, substitutionValue;
 	double maxScore = NEGATIVE_INFINITY;
 	if (scoreOnly) {
@@ -217,15 +309,15 @@ static double pairwiseAlignment(
 			CURR_MATRIX(0, 1) = PREV_MATRIX(0, 1) + endGapAddend;
 			CURR_MATRIX(0, 2) = NEGATIVE_INFINITY;
 
-			SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align2InfoPtr->string.seq[jElt]);
+			SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align2InfoPtr->string.ptr[jElt]);
 			stringElt2 = lookupValue;
-			SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence2.seq[scalar2 ? 0 : jElt]);
+			SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence2.ptr[scalar2 ? 0 : jElt]);
 			element2 = lookupValue;
 			if (localAlignment) {
 				for (i = 1, iMinus1 = 0, iElt = nCharString1Minus1; i <= nCharString1; i++, iMinus1++, iElt--) {
-					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.seq[iElt]);
+					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.ptr[iElt]);
 					stringElt1 = lookupValue;
-					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.seq[scalar1 ? 0 : iElt]);
+					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.ptr[scalar1 ? 0 : iElt]);
 					element1 = lookupValue;
 					fuzzy = FUZZY_MATRIX(stringElt1, stringElt2);
 					substitutionValue = (float) SUBSTITUTION_ARRAY(element1, element2, fuzzy);
@@ -235,19 +327,19 @@ static double pairwiseAlignment(
 							MAX(PREV_MATRIX(iMinus1, 0),
 							MAX(PREV_MATRIX(iMinus1, 1), PREV_MATRIX(iMinus1, 2))) + substitutionValue);
 					CURR_MATRIX(i, 1) =
-						MAX(MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) + gapOpeningPlusExtension,
-						    PREV_MATRIX(i, 1) + gapExtension);
+						MAX(MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) - gapOpeningPlusExtension,
+						    PREV_MATRIX(i, 1) - gapExtension);
 					CURR_MATRIX(i, 2) =
-						MAX(MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) + gapOpeningPlusExtension,
-						    CURR_MATRIX(iMinus1, 2) + gapExtension);
+						MAX(MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) - gapOpeningPlusExtension,
+						    CURR_MATRIX(iMinus1, 2) - gapExtension);
 
 					maxScore = MAX(CURR_MATRIX(i, 0), maxScore);
 				}
 			} else {
 				for (i = 1, iMinus1 = 0, iElt = nCharString1Minus1; i <= nCharString1; i++, iMinus1++, iElt--) {
-					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.seq[iElt]);
+					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.ptr[iElt]);
 					stringElt1 = lookupValue;
-					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.seq[scalar1 ? 0 : iElt]);
+					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.ptr[scalar1 ? 0 : iElt]);
 					element1 = lookupValue;
 					fuzzy = FUZZY_MATRIX(stringElt1, stringElt2);
 					substitutionValue = (float) SUBSTITUTION_ARRAY(element1, element2, fuzzy);
@@ -256,11 +348,11 @@ static double pairwiseAlignment(
 						MAX(PREV_MATRIX(iMinus1, 0),
 						MAX(PREV_MATRIX(iMinus1, 1), PREV_MATRIX(iMinus1, 2))) + substitutionValue;
 					CURR_MATRIX(i, 1) =
-						MAX(MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) + gapOpeningPlusExtension,
-						    PREV_MATRIX(i, 1) + gapExtension);
+						MAX(MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) - gapOpeningPlusExtension,
+						    PREV_MATRIX(i, 1) - gapExtension);
 					CURR_MATRIX(i, 2) =
-						MAX(MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) + gapOpeningPlusExtension,
-						    CURR_MATRIX(iMinus1, 2) + gapExtension);
+						MAX(MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) - gapOpeningPlusExtension,
+						    CURR_MATRIX(iMinus1, 2) - gapExtension);
 				}
 				if (noEndGap2) {
 					CURR_MATRIX(nCharString1, 1) =
@@ -316,15 +408,15 @@ static double pairwiseAlignment(
 			CURR_MATRIX(0, 1) = PREV_MATRIX(0, 1) + endGapAddend;
 			CURR_MATRIX(0, 2) = NEGATIVE_INFINITY;
 
-			SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align2InfoPtr->string.seq[jElt]);
+			SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align2InfoPtr->string.ptr[jElt]);
 			stringElt2 = lookupValue;
-			SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence2.seq[scalar2 ? 0 : jElt]);
+			SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence2.ptr[scalar2 ? 0 : jElt]);
 			element2 = lookupValue;
 			if (localAlignment) {
 				for (i = 1, iMinus1 = 0, iElt = nCharString1Minus1; i <= nCharString1; i++, iMinus1++, iElt--) {
-					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.seq[iElt]);
+					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.ptr[iElt]);
 					stringElt1 = lookupValue;
-					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.seq[scalar1 ? 0 : iElt]);
+					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.ptr[scalar1 ? 0 : iElt]);
 					element1 = lookupValue;
 					fuzzy = FUZZY_MATRIX(stringElt1, stringElt2);
 					substitutionValue = (float) SUBSTITUTION_ARRAY(element1, element2, fuzzy);
@@ -342,25 +434,25 @@ static double pairwiseAlignment(
 						S_TRACE_MATRIX(iMinus1, jMinus1) = INSERTION;
 						CURR_MATRIX(i, 0) = PREV_MATRIX(iMinus1, 2) + substitutionValue;
 					}
-					if (PREV_MATRIX(i, 1) > (MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) + gapOpening)) {
+					if (PREV_MATRIX(i, 1) > (MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) - gapOpening)) {
 						D_TRACE_MATRIX(iMinus1, jMinus1) = DELETION;
-						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 1) + gapExtension;
+						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 1) - gapExtension;
 					} else if (PREV_MATRIX(i, 0) >= PREV_MATRIX(i, 2)) {
 						D_TRACE_MATRIX(iMinus1, jMinus1) = SUBSTITUTION;
-						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 0) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 0) - gapOpeningPlusExtension;
 					} else {
 						D_TRACE_MATRIX(iMinus1, jMinus1) = INSERTION;
-						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 2) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 2) - gapOpeningPlusExtension;
 					}
-					if (CURR_MATRIX(iMinus1, 2) > (MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) + gapOpening)) {
+					if (CURR_MATRIX(iMinus1, 2) > (MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) - gapOpening)) {
 						I_TRACE_MATRIX(iMinus1, jMinus1) = INSERTION;
-						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 2) + gapExtension;
+						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 2) - gapExtension;
 					} else if (CURR_MATRIX(iMinus1, 0) >= CURR_MATRIX(iMinus1, 1)) {
 						I_TRACE_MATRIX(iMinus1, jMinus1) = SUBSTITUTION;
-						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 0) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 0) - gapOpeningPlusExtension;
 					} else {
 						I_TRACE_MATRIX(iMinus1, jMinus1) = DELETION;
-						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 1) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 1) - gapOpeningPlusExtension;
 					}
 
 					CURR_MATRIX(i, 0) = MAX(0.0, CURR_MATRIX(i, 0));
@@ -382,9 +474,9 @@ static double pairwiseAlignment(
 				}
 			} else {
 				for (i = 1, iMinus1 = 0, iElt = nCharString1Minus1; i <= nCharString1; i++, iMinus1++, iElt--) {
-					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.seq[iElt]);
+					SET_LOOKUP_VALUE(fuzzyLookupTable, fuzzyLookupTableLength, align1InfoPtr->string.ptr[iElt]);
 					stringElt1 = lookupValue;
-					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.seq[scalar1 ? 0 : iElt]);
+					SET_LOOKUP_VALUE(substitutionLookupTable, substitutionLookupTableLength, sequence1.ptr[scalar1 ? 0 : iElt]);
 					element1 = lookupValue;
 					fuzzy = FUZZY_MATRIX(stringElt1, stringElt2);
 					substitutionValue = (float) SUBSTITUTION_ARRAY(element1, element2, fuzzy);
@@ -402,25 +494,25 @@ static double pairwiseAlignment(
 						S_TRACE_MATRIX(iMinus1, jMinus1) = INSERTION;
 						CURR_MATRIX(i, 0) = PREV_MATRIX(iMinus1, 2) + substitutionValue;
 					}
-					if (PREV_MATRIX(i, 1) > (MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) + gapOpening)) {
+					if (PREV_MATRIX(i, 1) > (MAX(PREV_MATRIX(i, 0), PREV_MATRIX(i, 2)) - gapOpening)) {
 						D_TRACE_MATRIX(iMinus1, jMinus1) = DELETION;
-						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 1) + gapExtension;
+						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 1) - gapExtension;
 					} else if (PREV_MATRIX(i, 0) >= PREV_MATRIX(i, 2)) {
 						D_TRACE_MATRIX(iMinus1, jMinus1) = SUBSTITUTION;
-						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 0) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 0) - gapOpeningPlusExtension;
 					} else {
 						D_TRACE_MATRIX(iMinus1, jMinus1) = INSERTION;
-						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 2) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 1) = PREV_MATRIX(i, 2) - gapOpeningPlusExtension;
 					}
-					if (CURR_MATRIX(iMinus1, 2) > (MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) + gapOpening)) {
+					if (CURR_MATRIX(iMinus1, 2) > (MAX(CURR_MATRIX(iMinus1, 0), CURR_MATRIX(iMinus1, 1)) - gapOpening)) {
 						I_TRACE_MATRIX(iMinus1, jMinus1) = INSERTION;
-						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 2) + gapExtension;
+						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 2) - gapExtension;
 					} else if (CURR_MATRIX(iMinus1, 0) >= CURR_MATRIX(iMinus1, 1)) {
 						I_TRACE_MATRIX(iMinus1, jMinus1) = SUBSTITUTION;
-						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 0) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 0) - gapOpeningPlusExtension;
 					} else {
 						I_TRACE_MATRIX(iMinus1, jMinus1) = DELETION;
-						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 1) + gapOpeningPlusExtension;
+						CURR_MATRIX(i, 2) = CURR_MATRIX(iMinus1, 1) - gapOpeningPlusExtension;
 					}
 				}
 			}
@@ -477,78 +569,8 @@ static double pairwiseAlignment(
 		}
 
 		/* Step 4:  Traceback through the score matrices */
-		i = nCharString1 - align1InfoPtr->startRange;
-		j = nCharString2 - align2InfoPtr->startRange;
-		char prevTraceMatrix = '?';
-		while (currTraceMatrix != TERMINATION && i >= 0 && j >= 0) {
-			switch (currTraceMatrix) {
-	    		case INSERTION:
-	    			if (I_TRACE_MATRIX(i, j) != TERMINATION) {
-		    			if (j == nCharString2Minus1) {
-		    				align1InfoPtr->startRange++;
-		    			} else {
-		    				align1InfoPtr->widthRange++;
-		    				if (prevTraceMatrix != INSERTION) {
-		    					align2InfoPtr->startIndel[align2InfoPtr->lengthIndel] = nCharString2 - j;
-		    					align2InfoPtr->lengthIndel++;
-		    				}
-		    				align2InfoPtr->widthIndel[align2InfoPtr->lengthIndel - 1] += 1;
-		    			}
-	    			}
-	    			prevTraceMatrix = currTraceMatrix;
-					currTraceMatrix = I_TRACE_MATRIX(i, j);
-	    			i--;
-	    			break;
-	    		case DELETION:
-	    			if (D_TRACE_MATRIX(i, j) != TERMINATION) {
-		    			if (i == nCharString1Minus1) {
-		    				align2InfoPtr->startRange++;
-		    			} else {
-		    				align2InfoPtr->widthRange++;
-		    				if (prevTraceMatrix != DELETION) {
-		    					align1InfoPtr->startIndel[align1InfoPtr->lengthIndel] = nCharString1 - i;
-		    					align1InfoPtr->lengthIndel++;
-		    				}
-		    				align1InfoPtr->widthIndel[align1InfoPtr->lengthIndel - 1] += 1;
-		    			}
-	    			}
-	    			prevTraceMatrix = currTraceMatrix;
-					currTraceMatrix = D_TRACE_MATRIX(i, j);
-	    			j--;
-	    			break;
-	    		case SUBSTITUTION:
-				prevTraceMatrix = currTraceMatrix;
-				currTraceMatrix = S_TRACE_MATRIX(i, j);
-	    			if (currTraceMatrix != TERMINATION) {
-		    			align1InfoPtr->widthRange++;
-		    			align2InfoPtr->widthRange++;
-	    			}
-					if (currTraceMatrix != TERMINATION && align1InfoPtr->string.seq[nCharString1Minus1 - i] !=
-						align2InfoPtr->string.seq[nCharString2Minus1 - j]) {
-						align1InfoPtr->mismatch[align1InfoPtr->lengthMismatch] = nCharString1 - i;
-						align2InfoPtr->mismatch[align2InfoPtr->lengthMismatch] = nCharString2 - j;
-						align1InfoPtr->lengthMismatch++;
-						align2InfoPtr->lengthMismatch++;
-					}
-					i--;
-	    			j--;
-	    			break;
-	    		default:
-	    			error("unknown traceback code %d", currTraceMatrix);
-	    			break;
-			}
-		}
-
-		const int offset1 = align1InfoPtr->startRange - 1;
-		if (offset1 > 0 && align1InfoPtr->lengthIndel > 0) {
-			for (i = 0; i < align1InfoPtr->lengthIndel; i++)
-				align1InfoPtr->startIndel[i] -= offset1;
-		}
-		const int offset2 = align2InfoPtr->startRange - 1;
-		if (offset2 > 0 && align2InfoPtr->lengthIndel > 0) {
-			for (j = 0; j < align2InfoPtr->lengthIndel; j++)
-				align2InfoPtr->startIndel[j] -= offset2;
-		}
+		traceback(alignBufferPtr, currTraceMatrix,
+			  align1InfoPtr, align2InfoPtr);
 	}
 
 	return (double) maxScore;
@@ -570,9 +592,9 @@ static double pairwiseAlignment(
  *                           of the optimal pairwise alignment
  *                           (logical vector of length 1)
  * 'gapOpening':             gap opening cost or penalty
- *                           (double vector of length 1)
+ *                           (single non-negative double)
  * 'gapExtension':           gap extension cost or penalty
- *                           (double vector of length 1)
+ *                           (single non-negative double)
  * 'useQuality':             denotes whether or not to use quality measures
  *                           in the optimal pairwise alignment
  *                           (logical vector of length 1)
@@ -619,9 +641,9 @@ SEXP XStringSet_align_pairwiseAlignment(
 	const int localAlignment = (INTEGER(typeCode)[0] == LOCAL_ALIGNMENT);
 	float gapOpeningValue = REAL(gapOpening)[0];
 	float gapExtensionValue = REAL(gapExtension)[0];
-	if (gapOpeningValue == NEGATIVE_INFINITY || gapExtensionValue == NEGATIVE_INFINITY) {
+	if (gapOpeningValue == POSITIVE_INFINITY || gapExtensionValue == POSITIVE_INFINITY) {
 		gapOpeningValue = 0.0;
-		gapExtensionValue = NEGATIVE_INFINITY;
+		gapExtensionValue = POSITIVE_INFINITY;
 	}
 
 	XStringSet_holder pattern_holder = _hold_XStringSet(pattern);
@@ -664,21 +686,26 @@ SEXP XStringSet_align_pairwiseAlignment(
 	/* Create the alignment buffer object */
 	struct AlignBuffer alignBuffer;
 	int nCharString1 = 0, nCharString2 = 0, nCharProduct = 0;
+	reset_ovflow_flag();
 	if (multipleSubjects) {
 		for (i = 0; i < numberOfStrings; i++) {
 			int nchar1 = _get_elt_from_XStringSet_holder(&pattern_holder, i).length;
 			int nchar2 = _get_elt_from_XStringSet_holder(&subject_holder, i).length;
 			nCharString1 = MAX(nCharString1, nchar1);
 			nCharString2 = MAX(nCharString2, nchar2);
-			nCharProduct = MAX(nCharProduct, nchar1 * nchar2);
+			nCharProduct = MAX(nCharProduct,
+					   safe_int_mult(nchar1, nchar2));
 		}
 	} else {
 		for (i = 0; i < numberOfStrings; i++) {
 			nCharString1 = MAX(nCharString1, _get_elt_from_XStringSet_holder(&pattern_holder, i).length);
 		}
 		nCharString2 = align2Info.string.length;
-		nCharProduct = nCharString1 * nCharString2;
+		nCharProduct = safe_int_mult(nCharString1, nCharString2);
 	}
+	if (get_ovflow_flag())
+		error("max(nchar(pattern) * nchar(subject)) is too big "
+		      "(must be <= %d)", INT_MAX);
 	const int alignmentBufferSize = nCharString1 + 1;
 	alignBuffer.currMatrix = (float *) R_alloc((long) 3 * alignmentBufferSize, sizeof(float));
 	alignBuffer.prevMatrix = (float *) R_alloc((long) 3 * alignmentBufferSize, sizeof(float));
@@ -1023,9 +1050,9 @@ SEXP XStringSet_align_pairwiseAlignment(
  *                             (integer vector of length 1;
  *                              1 = 'global', 2 = 'local', 3 = 'overlap')
  * 'gapOpening':               gap opening cost or penalty
- *                             (double vector of length 1)
+ *                             (single non-negative double)
  * 'gapExtension':             gap extension cost or penalty
- *                             (double vector of length 1)
+ *                             (single non-negative double)
  * 'useQuality':               denotes whether or not to use quality measures
  *                             in the optimal pairwise alignment
  *                             (logical vector of length 1)
@@ -1067,9 +1094,9 @@ SEXP XStringSet_align_distance(
 	int useQualityValue = LOGICAL(useQuality)[0];
 	float gapOpeningValue = REAL(gapOpening)[0];
 	float gapExtensionValue = REAL(gapExtension)[0];
-	if (gapOpeningValue == NEGATIVE_INFINITY || gapExtensionValue == NEGATIVE_INFINITY) {
+	if (gapOpeningValue == POSITIVE_INFINITY || gapExtensionValue == POSITIVE_INFINITY) {
 		gapOpeningValue = 0.0;
-		gapExtensionValue = NEGATIVE_INFINITY;
+		gapExtensionValue = POSITIVE_INFINITY;
 	}
 	int localAlignment = (INTEGER(typeCode)[0] == LOCAL_ALIGNMENT);
 
